package com.sunmi.cloudprint;

import androidx.appcompat.app.AppCompatActivity;

import android.os.Bundle;
import android.util.Log;
import android.view.View;

import com.sunmi.externalprinterlibrary.api.ConnectCallback;
import com.sunmi.externalprinterlibrary.api.PrinterException;
import com.sunmi.externalprinterlibrary.api.Status;
import com.sunmi.externalprinterlibrary.api.SunmiPrinterApi;


import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

public class MainActivity extends AppCompatActivity {

    public static final String TAG = "Demo";

    ExecutorService service;
    boolean isConnect;


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        service = Executors.newSingleThreadExecutor();
    }

    /**
     * 先连接打印机确认打印机的连接状态
     * 使用局域网打印需要调用connectNetPrinter方法
     */
    public void init(View view) {
        try {
            SunmiPrinterApi.getInstance().connectNetPrinter(getApplicationContext(), "10.10.128.255", new ConnectCallback() {
                @Override
                public void onFound() {

                }

                @Override
                public void onUnfound() {

                }

                @Override
                public void onConnect() {
                    isConnect = true;
                }

                @Override
                public void onDisconnect() {
                    isConnect = false;
                }
            });
        } catch (PrinterException e) {
            e.printStackTrace();
        }
    }

    /**
     * 当打印结束后需要释放打印机
     * 对于局域网打印，如果有多个应用同时打印，建议每次打印结束都释放打印机确保打印机资源可被其他应用使用
     */
    public void deInit(View view) {
        try {
            SunmiPrinterApi.getInstance().disconnectPrinter(this);
        } catch (PrinterException e) {
            e.printStackTrace();
        }
        isConnect = false;
    }

    /**
     * 普通打印一单小票
     * 普通打印不关心打印执行结果
     * 建议打印接口调用在非主线程（网络打印机必须在非主线程）
     */
    public void simplePrint1(View view) {
        if(!isConnect){
            return;
        }
        service.execute(() -> {
            try {
                int status = SunmiPrinterApi.getInstance().getPrinterStatus();
                if(status != Status.RUNNING){
                    Log.d(TAG, "打印机状态异常->" + status) ;
                }else{
                    printMeituanBill();
                }
            } catch (PrinterException e) {
                e.printStackTrace();
            }
        });

    }

    /**
     * 普通打印一单小票
     * 普通打印不关心打印执行结果
     * 建议打印接口调用在非主线程（网络打印机必须在非主线程）
     */
    public void simplePrint2(View view) {
        if(!isConnect){
            return;
        }
        service.execute(() -> {
            try {
                int status = SunmiPrinterApi.getInstance().getPrinterStatus();
                if(status != Status.RUNNING){
                    Log.d(TAG, "打印机状态异常->" + status) ;
                }else{
                    SunmiPrinterApi.getInstance().sendRawData(getMeituanBill());
                }
            } catch (PrinterException e) {
                e.printStackTrace();
            }
        });
    }

    /**
     * 事务打印基础方法
     * 启用事务->提交事务->结束事务
     */
    public void transPrint1(View view) {
        service.execute(() -> {
            try {
                SunmiPrinterApi.getInstance().startTransBuffer(true);
                printMeituanBill();
                SunmiPrinterApi.getInstance().commitTransBuffer((b, i) -> {
                    if(b){
                        Log.d(TAG, "transPrint1: 打印成功");
                    }else{
                        Log.d(TAG, "transPrint1: 打印失败");
                    }
                    try {
                        SunmiPrinterApi.getInstance().endTransBuffer();
                    } catch (PrinterException e) {
                        e.printStackTrace();
                    }
                });
            } catch (PrinterException e) {
                e.printStackTrace();
            }
        });
    }

    /**
     * 事务打印连续打印
     * 启用事务->提交事务->打印失败->自动重打->结束事务
     */
    public void transPrint2(View view) {
        service.execute(() -> {
            try {
                SunmiPrinterApi.getInstance().startTransBuffer(true);
                printMeituanBill();
                SunmiPrinterApi.getInstance().commitTransBuffer((b, i) -> {
                    //假设打印失败
                    transPrint1(null);
                });
            } catch (PrinterException e) {
                e.printStackTrace();
            }
        });
    }

    /**
     * 事务打印必须被提交，否则将缓存在内存中，若未提交直接退出事务，所有内容将丢弃
     * 以下将打印两单内容
     */
    public void transPrint3(View view) {
        service.execute(() -> {
            try {
                SunmiPrinterApi.getInstance().startTransBuffer(true);
                printMeituanBill();
                printMeituanBill();
                printMeituanBill();
                SunmiPrinterApi.getInstance().endTransBuffer();
                SunmiPrinterApi.getInstance().startTransBuffer(true);
                printMeituanBill();
                SunmiPrinterApi.getInstance().startTransBuffer(false);
                printMeituanBill();
                SunmiPrinterApi.getInstance().commitTransBuffer(null);
                SunmiPrinterApi.getInstance().endTransBuffer();
            } catch (PrinterException e) {
                e.printStackTrace();
            }
        });
    }

    /**
     * 接口-美团小票
     */
    private void printMeituanBill() throws PrinterException {
        SunmiPrinterApi.getInstance().printerInit();
        SunmiPrinterApi.getInstance().setAlignMode(1);
        SunmiPrinterApi.getInstance().setFontZoom(2, 2);
        SunmiPrinterApi.getInstance().printText("#2 美团测试");
        SunmiPrinterApi.getInstance().flush();
        SunmiPrinterApi.getInstance().setFontZoom(1, 1);
        SunmiPrinterApi.getInstance().printText("粤香港式烧腊（第1联）");
        SunmiPrinterApi.getInstance().flush();
        SunmiPrinterApi.getInstance().printText("--------------------------------");
        SunmiPrinterApi.getInstance().flush();
        SunmiPrinterApi.getInstance().setFontZoom(1, 2);
        SunmiPrinterApi.getInstance().printText("* * * * * *  预订单  * * * * * *");
        SunmiPrinterApi.getInstance().flush();
        SunmiPrinterApi.getInstance().printText("期望送达时间:[18:00]");
        SunmiPrinterApi.getInstance().flush();
        SunmiPrinterApi.getInstance().printerInit();
        SunmiPrinterApi.getInstance().printText("--------------------------------");
        SunmiPrinterApi.getInstance().printText("下单时间：01-01 12：00");
        SunmiPrinterApi.getInstance().flush();
        SunmiPrinterApi.getInstance().setFontZoom(1, 2);
        SunmiPrinterApi.getInstance().printText("备注：别太辣");
        SunmiPrinterApi.getInstance().flush();
        SunmiPrinterApi.getInstance().setFontZoom(1, 1);
        SunmiPrinterApi.getInstance().printText("--------------------------------");
        SunmiPrinterApi.getInstance().flush();
        SunmiPrinterApi.getInstance().printText("菜名         数量     小计");
        SunmiPrinterApi.getInstance().flush();
        SunmiPrinterApi.getInstance().printText("--------------------------------");
        SunmiPrinterApi.getInstance().flush();
        SunmiPrinterApi.getInstance().setFontZoom(1, 2);
        SunmiPrinterApi.getInstance().printText("红烧肉        X1       12");
        SunmiPrinterApi.getInstance().flush();
        SunmiPrinterApi.getInstance().setFontZoom(1, 1);
        SunmiPrinterApi.getInstance().printText("--------------------------------");
        SunmiPrinterApi.getInstance().flush();
        SunmiPrinterApi.getInstance().printText("配送费                    5");
        SunmiPrinterApi.getInstance().flush();
        SunmiPrinterApi.getInstance().printText("餐盒费                    1");
        SunmiPrinterApi.getInstance().flush();
        SunmiPrinterApi.getInstance().printText("[超时赔付] -详见订单");
        SunmiPrinterApi.getInstance().flush();
        SunmiPrinterApi.getInstance().printText("可口可乐：x1");
        SunmiPrinterApi.getInstance().flush();
        SunmiPrinterApi.getInstance().printText("--------------------------------");
        SunmiPrinterApi.getInstance().flush();
        SunmiPrinterApi.getInstance().setFontZoom(1, 2);
        SunmiPrinterApi.getInstance().printText("合计                    18元");
        SunmiPrinterApi.getInstance().flush();
        SunmiPrinterApi.getInstance().setFontZoom(1, 1);
        SunmiPrinterApi.getInstance().printText("--------------------------------");
        SunmiPrinterApi.getInstance().flush();
        SunmiPrinterApi.getInstance().setFontZoom(2, 2);
        SunmiPrinterApi.getInstance().printText("张* 18312345678地址信息");
        SunmiPrinterApi.getInstance().flush();
        SunmiPrinterApi.getInstance().setFontZoom(1, 1);
        SunmiPrinterApi.getInstance().printText("--------------------------------");
        SunmiPrinterApi.getInstance().flush();
        SunmiPrinterApi.getInstance().lineWrap(1);
        SunmiPrinterApi.getInstance().setFontZoom(2, 2);
        SunmiPrinterApi.getInstance().printText("#2 美团测试");
        SunmiPrinterApi.getInstance().lineWrap(5);
    }


    /**
     * ESC指令-美团小票
     */
    public static byte[] getMeituanBill() {
        byte[] rv = new byte[]{
                0x1b, 0x40, 0x1b, 0x61, 0x01, 0x1d, 0x21, 0x11, (byte) 0xa3, (byte) 0xa3, 0x31, 0x20, 0x20, (byte) 0xc3, (byte) 0xc0, (byte) 0xcd, (byte) 0xc5, (byte) 0xb2, (byte) 0xe2, (byte) 0xca, (byte) 0xd4, 0x0a
                , 0x0a, 0x1d, 0x21, 0x00, (byte) 0xd4, (byte) 0xc1, (byte) 0xcf, (byte) 0xe3, (byte) 0xb8, (byte) 0xdb, (byte) 0xca, (byte) 0xbd, (byte) 0xc9, (byte) 0xd5, (byte) 0xc0, (byte) 0xb0, 0x28, (byte) 0xb5, (byte) 0xda, 0x31, (byte) 0xc1, (byte) 0xaa
                , 0x29, 0x0a, 0x1b, 0x21, 0x10, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d
                , 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x0a, 0x2a, 0x20, 0x2a, 0x20, 0x2a, 0x20
                , 0x2a, 0x20, 0x2a, 0x20, 0x2a, 0x20, 0x20, (byte) 0xd4, (byte) 0xa4, (byte) 0xb6, (byte) 0xa9, (byte) 0xb5, (byte) 0xa5, 0x20, 0x20, 0x2a, 0x20, 0x2a, 0x20, 0x2a, 0x20, 0x2a
                , 0x20, 0x2a, 0x20, 0x2a, 0x0a, (byte) 0xc6, (byte) 0xda, (byte) 0xcd, (byte) 0xfb, (byte) 0xcb, (byte) 0xcd, (byte) 0xb4, (byte) 0xef, (byte) 0xca, (byte) 0xb1, (byte) 0xbc, (byte) 0xe4, 0x3a, 0x20, 0x5b, 0x31, 0x38
                , 0x3a, 0x30, 0x30, 0x5d, 0x0a, 0x1d, 0x21, 0x00, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d
                , 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x0a, 0x1b, 0x61, 0x00
                , (byte) 0xcf, (byte) 0xc2, (byte) 0xb5, (byte) 0xa5, (byte) 0xca, (byte) 0xb1, (byte) 0xbc, (byte) 0xe4, 0x3a, 0x30, 0x31, 0x2d, 0x30, 0x31, 0x20, 0x31, 0x32, 0x3a, 0x30, 0x30, 0x0a, 0x1b
                , 0x21, 0x10, (byte) 0xb1, (byte) 0xb8, (byte) 0xd7, (byte) 0xa2, 0x3a, (byte) 0xb1, (byte) 0xf0, (byte) 0xcc, (byte) 0xab, (byte) 0xc0, (byte) 0xb1, 0x0a, 0x1d, 0x21, 0x00, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d
                , 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d
                , 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x0a, (byte) 0xb2, (byte) 0xcb, (byte) 0xc3, (byte) 0xfb, 0x09, 0x09, 0x20, 0x20, 0x20, (byte) 0xca, (byte) 0xfd, (byte) 0xc1, (byte) 0xbf, 0x09, 0x20, 0x20
                , 0x20, 0x20, (byte) 0xd0, (byte) 0xa1, (byte) 0xbc, (byte) 0xc6, 0x09, 0x0a, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d
                , 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x0a, 0x1b, 0x21, 0x10
                , (byte) 0xba, (byte) 0xec, (byte) 0xc9, (byte) 0xd5, (byte) 0xc8, (byte) 0xe2, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x78
                , 0x31, 0x09, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x31, 0x32, 0x0a, 0x1d, 0x21, 0x00, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d
                , 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d
                , 0x2d, 0x2d, 0x0a, (byte) 0xc5, (byte) 0xe4, (byte) 0xcb, (byte) 0xcd, (byte) 0xb7, (byte) 0xd1, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20
                , 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x35, 0x0a, (byte) 0xb2, (byte) 0xcd, (byte) 0xba, (byte) 0xd0, (byte) 0xb7, (byte) 0xd1, 0x20, 0x20
                , 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20
                , 0x20, 0x31, 0x0a, 0x5b, (byte) 0xb3, (byte) 0xac, (byte) 0xca, (byte) 0xb1, (byte) 0xc5, (byte) 0xe2, (byte) 0xb8, (byte) 0xb6, 0x5d, 0x20, 0x2d, (byte) 0xcf, (byte) 0xea, (byte) 0xbc, (byte) 0xfb, (byte) 0xb6, (byte) 0xa9, (byte) 0xb5
                , (byte) 0xa5, 0x0a, (byte) 0xbf, (byte) 0xc9, (byte) 0xbf, (byte) 0xda, (byte) 0xbf, (byte) 0xc9, (byte) 0xc0, (byte) 0xd6, 0x3a, 0x78, 0x31, 0x0a, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d
                , 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d
                , 0x2d, 0x2d, 0x0a, 0x1b, 0x21, 0x10, (byte) 0xba, (byte) 0xcf, (byte) 0xbc, (byte) 0xc6, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20
                , 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x31, 0x38, (byte) 0xd4, (byte) 0xaa, 0x0a, 0x1b, 0x40, 0x2d, 0x2d, 0x2d
                , 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d
                , 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x0a, 0x1d, 0x21, 0x11, (byte) 0xd5, (byte) 0xc5, 0x2a, 0x20, 0x31, 0x38, 0x33, 0x31, 0x32, 0x33, 0x34
                , 0x35, 0x36, 0x37, 0x38, 0x0a, (byte) 0xb5, (byte) 0xd8, (byte) 0xd6, (byte) 0xb7, (byte) 0xd0, (byte) 0xc5, (byte) 0xcf, (byte) 0xa2, 0x0a, 0x1d, 0x21, 0x00, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d
                , 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x2d
                , 0x2d, 0x2d, 0x2d, 0x2d, 0x2d, 0x0a, 0x0a, 0x1b, 0x40, 0x1b, 0x61, 0x01, 0x1d, 0x21, 0x11, (byte) 0xa3, (byte) 0xa3, 0x31, 0x20, 0x20, (byte) 0xc3, (byte) 0xc0
                , (byte) 0xcd, (byte) 0xc5, (byte) 0xb2, (byte) 0xe2, (byte) 0xca, (byte) 0xd4, 0x0a, 0x1d, 0x21, 0x00, 0x1b, 0x40, 0x0a, 0x0a, 0x0a, 0x0a
        };
        return rv;
    }
}